#!/bin/bash
# This script receives LoRa data from the Arduino MCU, does some
# processing, and passes the data on to an IoTool server via MQTT.
#
# Install the mosquitto_client first with the package manager.
# Then copy this script to the /root folder. You may copy it
# elsewhere, but don't forget to also change to location in
# lora_server.pde.
#
# Made by David Sluga <david.sluga@senlab.io> for
# SenLab d.o.o.
#

# Get saved settings from OpenWrt configuration.
SERVER=$(uci get mqtt.general.server)
PORT=$(uci get mqtt.general.port)
PUBTOPIC=$(uci get mqtt.general.pub_topic)
DEVID=$(uci get mqtt.general.client_id)
USERNAME=$(uci get mqtt.general.user_name)
PASSWORD=$(uci get mqtt.general.password)

# Listen for arguments provided when launching the script.
while getopts n:s:r:t:d opts; do
	case ${opts} in
		n) NODE=${OPTARG} ;;
		s) SENSOR=${OPTARG} ;;
		r) READING=${OPTARG} ;;
        t) TIMESTAMP=${OPTARG} ;;
        d) DEBUG=1 ;;
	esac
done

# Check if all necessary arguments have been provided. If not, exit the script.
if [[ -z $NODE || -z $SENSOR || -z $READING || -z $TIMESTAMP ]]; then
	echo -e "Some or all of required arguments are missing, please provide all arguments and run the script again.\n"
	echo -e "Usage: iotoolmqtt.sh -n <node_id> -s <sensor_id> -r <reading_value> -t <reading_timestamp>\n"
	echo -e "Optional arguments:\n"
	echo "  -d			enable MQTT debug"
	exit 1
fi

# Predefined list of node ID and name combinations. Since we control all the nodes it's not a problem.
case $NODE in
    2) NODENAME="arduino1" ;;
    3) NODENAME="arduino2" ;;
    *) NODENAME=$NODE ;;
esac

# Predefined list of sensor ID and name combinations. Since we control all the nodes and the data they send it's not a problem.
case $SENSOR in
    3) SENSORNAME="TemperatureC" ;;
    4) SENSORNAME="Humidity" ;;
    5) SENSORNAME="Pressure" ;;
    1) SENSORNAME="GPSLatitude" ;;
    2) SENSORNAME="GPSLongitude" ;;
    *) SENSORNAME=$SENSOR ;;
esac

# In case we have the debug flag enabled and SSL port (8883) configured, provide the debug parameter and OpenWrt SSL directory to mosquitto_pub client.
if [[ -v $DEBUG && "$PORT" == 8883 ]]; then
mosquitto_pub -d -h $SERVER -p $PORT -i $DEVID -q 1 -u iotool_pub -P publishiotool -t $PUBTOPIC -m '[{"username":"'"$USERNAME"'"},{"password":"'"$PASSWORD"'"},{"sensor":"'"$SENSORNAME"'@'"$NODENAME"'","reading":"'"$READING"'","datetimems":"'"$TIMESTAMP"'000"}]' --capath /etc/ssl/certs
# In case we have just the debug flag provided to the script, provide the debug parameter to mosquitto_pub client.
elif [[ -v $DEBUG ]]; then
mosquitto_pub -d -h $SERVER -p $PORT -i $DEVID -q 1 -u iotool_pub -P publishiotool -t $PUBTOPIC -m '[{"username":"'"$USERNAME"'"},{"password":"'"$PASSWORD"'"},{"sensor":"'"$SENSORNAME"'@'"$NODENAME"'","reading":"'"$READING"'","datetimems":"'"$TIMESTAMP"'000"}]'
# In case we have just the SSL port configured, provide the OpenWrt SSL directory to mosquitto_pub client.
elif [[ "$PORT" == 8883 ]]; then
mosquitto_pub -h $SERVER -p $PORT -i $DEVID -q 1 -u iotool_pub -P publishiotool -t $PUBTOPIC -m '[{"username":"'"$USERNAME"'"},{"password":"'"$PASSWORD"'"},{"sensor":"'"$SENSORNAME"'@'"$NODENAME"'","reading":"'"$READING"'","datetimems":"'"$TIMESTAMP"'000"}]' --capath /etc/ssl/certs
# In case there are no special settings, skip all extra flags.
else
mosquitto_pub -h $SERVER -p $PORT -i $DEVID -q 1 -u iotool_pub -P publishiotool -t $PUBTOPIC -m '[{"username":"'"$USERNAME"'"},{"password":"'"$PASSWORD"'"},{"sensor":"'"$SENSORNAME"'@'"$NODENAME"'","reading":"'"$READING"'","datetimems":"'"$TIMESTAMP"'000"}]'
fi